@@ -7,6 +7,7 @@ package github
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -143,6 +144,50 @@ func TestActionsService_GetWorkflowRunByID(t *testing.T) {
 	})
 }
 
+func TestActionsService_GetWorkflowRunAttempt(t *testing.T) {
+	client, mux, _, teardown := setup()
+	defer teardown()
+
+	mux.HandleFunc("/repos/o/r/actions/runs/29679449/attempts/3", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, "GET")
+		testFormValues(t, r, values{"exclude_pull_requests": "true"})
+		fmt.Fprint(w, `{"id":399444496,"run_number":296,"run_attempt":3,"created_at":"2019-01-02T15:04:05Z","updated_at":"2020-01-02T15:04:05Z"}}`)
+	})
+
+	opts := &WorkflowRunAttemptOptions{ExcludePullRequests: Bool(true)}
+	ctx := context.Background()
+	runs, _, err := client.Actions.GetWorkflowRunAttempt(ctx, "o", "r", 29679449, 3, opts)
+	if err != nil {
+		t.Errorf("Actions.GetWorkflowRunAttempt returned error: %v", err)
+	}
+
+	want := &WorkflowRun{
+		ID:         Int64(399444496),
+		RunNumber:  Int(296),
+		RunAttempt: Int(3),
+		CreatedAt:  &Timestamp{time.Date(2019, time.January, 02, 15, 04, 05, 0, time.UTC)},
+		UpdatedAt:  &Timestamp{time.Date(2020, time.January, 02, 15, 04, 05, 0, time.UTC)},
+	}
+
+	if !cmp.Equal(runs, want) {
+		t.Errorf("Actions.GetWorkflowRunAttempt returned %+v, want %+v", runs, want)
+	}
+
+	const methodName = "GetWorkflowRunAttempt"
+	testBadOptions(t, methodName, func() (err error) {
+		_, _, err = client.Actions.GetWorkflowRunAttempt(ctx, "\n", "\n", 29679449, 3, opts)
+		return err
+	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		got, resp, err := client.Actions.GetWorkflowRunAttempt(ctx, "o", "r", 29679449, 3, opts)
+		if got != nil {
+			t.Errorf("testNewRequestAndDoFailure %v = %#v, want nil", methodName, got)
+		}
+		return resp, err
+	})
+}
+
 func TestActionsService_RerunWorkflowRunByID(t *testing.T) {
 	client, mux, _, teardown := setup()
 	defer teardown()
@@ -172,6 +217,64 @@ func TestActionsService_RerunWorkflowRunByID(t *testing.T) {
 	})
 }
 
+func TestActionsService_RerunFailedJobsByID(t *testing.T) {
+	client, mux, _, teardown := setup()
+	defer teardown()
+
+	mux.HandleFunc("/repos/o/r/actions/runs/3434/rerun-failed-jobs", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, "POST")
+		w.WriteHeader(http.StatusCreated)
+	})
+
+	ctx := context.Background()
+	resp, err := client.Actions.RerunFailedJobsByID(ctx, "o", "r", 3434)
+	if err != nil {
+		t.Errorf("Actions.RerunFailedJobsByID returned error: %v", err)
+	}
+	if resp.StatusCode != http.StatusCreated {
+		t.Errorf("Actions.RerunFailedJobsByID returned status: %d, want %d", resp.StatusCode, http.StatusCreated)
+	}
+
+	const methodName = "RerunFailedJobsByID"
+	testBadOptions(t, methodName, func() (err error) {
+		_, err = client.Actions.RerunFailedJobsByID(ctx, "\n", "\n", 3434)
+		return err
+	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		return client.Actions.RerunFailedJobsByID(ctx, "o", "r", 3434)
+	})
+}
+
+func TestActionsService_RerunJobByID(t *testing.T) {
+	client, mux, _, teardown := setup()
+	defer teardown()
+
+	mux.HandleFunc("/repos/o/r/actions/jobs/3434/rerun", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, "POST")
+		w.WriteHeader(http.StatusCreated)
+	})
+
+	ctx := context.Background()
+	resp, err := client.Actions.RerunJobByID(ctx, "o", "r", 3434)
+	if err != nil {
+		t.Errorf("Actions.RerunJobByID returned error: %v", err)
+	}
+	if resp.StatusCode != http.StatusCreated {
+		t.Errorf("Actions.RerunJobByID returned status: %d, want %d", resp.StatusCode, http.StatusCreated)
+	}
+
+	const methodName = "RerunJobByID"
+	testBadOptions(t, methodName, func() (err error) {
+		_, err = client.Actions.RerunJobByID(ctx, "\n", "\n", 3434)
+		return err
+	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		return client.Actions.RerunJobByID(ctx, "o", "r", 3434)
+	})
+}
+
 func TestActionsService_CancelWorkflowRunByID(t *testing.T) {
 	client, mux, _, teardown := setup()
 	defer teardown()
@@ -295,7 +398,6 @@ func TestActionService_ListRepositoryWorkflowRuns(t *testing.T) {
 		"workflow_runs":[
 			{"id":298499444,"run_number":301,"created_at":"2020-04-11T11:14:54Z","updated_at":"2020-04-11T11:14:54Z"},
 			{"id":298499445,"run_number":302,"created_at":"2020-04-11T11:14:54Z","updated_at":"2020-04-11T11:14:54Z"}]}`)
-
 	})
 
 	opts := &ListWorkflowRunsOptions{ListOptions: ListOptions{Page: 2, PerPage: 2}}
@@ -335,6 +437,32 @@ func TestActionService_ListRepositoryWorkflowRuns(t *testing.T) {
 	})
 }
 
+func TestActionService_DeleteWorkflowRun(t *testing.T) {
+	client, mux, _, teardown := setup()
+	defer teardown()
+
+	mux.HandleFunc("/repos/o/r/actions/runs/399444496", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, "DELETE")
+
+		w.WriteHeader(http.StatusNoContent)
+	})
+
+	ctx := context.Background()
+	if _, err := client.Actions.DeleteWorkflowRun(ctx, "o", "r", 399444496); err != nil {
+		t.Errorf("DeleteWorkflowRun returned error: %v", err)
+	}
+
+	const methodName = "DeleteWorkflowRun"
+	testBadOptions(t, methodName, func() (err error) {
+		_, err = client.Actions.DeleteWorkflowRun(ctx, "\n", "\n", 399444496)
+		return err
+	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		return client.Actions.DeleteWorkflowRun(ctx, "o", "r", 399444496)
+	})
+}
+
 func TestActionService_DeleteWorkflowRunLogs(t *testing.T) {
 	client, mux, _, teardown := setup()
 	defer teardown()
@@ -367,7 +495,7 @@ func TestActionsService_GetWorkflowRunUsageByID(t *testing.T) {
 
 	mux.HandleFunc("/repos/o/r/actions/runs/29679449/timing", func(w http.ResponseWriter, r *http.Request) {
 		testMethod(t, r, "GET")
-		fmt.Fprint(w, `{"billable":{"UBUNTU":{"total_ms":180000,"jobs":1},"MACOS":{"total_ms":240000,"jobs":4},"WINDOWS":{"total_ms":300000,"jobs":2}},"run_duration_ms":500000}`)
+		fmt.Fprint(w, `{"billable":{"UBUNTU":{"total_ms":180000,"jobs":1,"job_runs":[{"job_id":1,"duration_ms":60000}]},"MACOS":{"total_ms":240000,"jobs":2,"job_runs":[{"job_id":2,"duration_ms":30000},{"job_id":3,"duration_ms":10000}]},"WINDOWS":{"total_ms":300000,"jobs":2}},"run_duration_ms":500000}`)
 	})
 
 	ctx := context.Background()
@@ -381,10 +509,26 @@ func TestActionsService_GetWorkflowRunUsageByID(t *testing.T) {
 			Ubuntu: &WorkflowRunBill{
 				TotalMS: Int64(180000),
 				Jobs:    Int(1),
+				JobRuns: []*WorkflowRunJobRun{
+					{
+						JobID:      Int(1),
+						DurationMS: Int64(60000),
+					},
+				},
 			},
 			MacOS: &WorkflowRunBill{
 				TotalMS: Int64(240000),
-				Jobs:    Int(4),
+				Jobs:    Int(2),
+				JobRuns: []*WorkflowRunJobRun{
+					{
+						JobID:      Int(2),
+						DurationMS: Int64(30000),
+					},
+					{
+						JobID:      Int(3),
+						DurationMS: Int64(10000),
+					},
+				},
 			},
 			Windows: &WorkflowRunBill{
 				TotalMS: Int64(300000),
@@ -423,6 +567,7 @@ func TestWorkflowRun_Marshal(t *testing.T) {
 		HeadBranch: String("hb"),
 		HeadSHA:    String("hs"),
 		RunNumber:  Int(1),
+		RunAttempt: Int(1),
 		Event:      String("e"),
 		Status:     String("s"),
 		Conclusion: String("c"),
@@ -454,14 +599,16 @@ func TestWorkflowRun_Marshal(t *testing.T) {
 				},
 			},
 		},
-		CreatedAt:     &Timestamp{referenceTime},
-		UpdatedAt:     &Timestamp{referenceTime},
-		JobsURL:       String("j"),
-		LogsURL:       String("l"),
-		CheckSuiteURL: String("c"),
-		ArtifactsURL:  String("a"),
-		CancelURL:     String("c"),
-		RerunURL:      String("r"),
+		CreatedAt:          &Timestamp{referenceTime},
+		UpdatedAt:          &Timestamp{referenceTime},
+		RunStartedAt:       &Timestamp{referenceTime},
+		JobsURL:            String("j"),
+		LogsURL:            String("l"),
+		CheckSuiteURL:      String("c"),
+		ArtifactsURL:       String("a"),
+		CancelURL:          String("c"),
+		RerunURL:           String("r"),
+		PreviousAttemptURL: String("p"),
 		HeadCommit: &HeadCommit{
 			Message: String("m"),
 			Author: &CommitAuthor{
@@ -492,6 +639,26 @@ func TestWorkflowRun_Marshal(t *testing.T) {
 			URL:  String("u"),
 			Name: String("n"),
 		},
+		Actor: &User{
+			Login:           String("l"),
+			ID:              Int64(1),
+			AvatarURL:       String("a"),
+			GravatarID:      String("g"),
+			Name:            String("n"),
+			Company:         String("c"),
+			Blog:            String("b"),
+			Location:        String("l"),
+			Email:           String("e"),
+			Hireable:        Bool(true),
+			Bio:             String("b"),
+			TwitterUsername: String("t"),
+			PublicRepos:     Int(1),
+			Followers:       Int(1),
+			Following:       Int(1),
+			CreatedAt:       &Timestamp{referenceTime},
+			SuspendedAt:     &Timestamp{referenceTime},
+			URL:             String("u"),
+		},
 	}
 
 	want := `{
@@ -501,6 +668,7 @@ func TestWorkflowRun_Marshal(t *testing.T) {
 		"head_branch": "hb",
 		"head_sha": "hs",
 		"run_number": 1,
+		"run_attempt": 1,
 		"event": "e",
 		"status": "s",
 		"conclusion": "c",
@@ -534,12 +702,14 @@ func TestWorkflowRun_Marshal(t *testing.T) {
 		],
 		"created_at": ` + referenceTimeStr + `,
 		"updated_at": ` + referenceTimeStr + `,
+		"run_started_at": ` + referenceTimeStr + `,
 		"jobs_url": "j",
 		"logs_url": "l",
 		"check_suite_url": "c",
 		"artifacts_url": "a",
 		"cancel_url": "c",
 		"rerun_url": "r",
+		"previous_attempt_url": "p",
 		"head_commit": {
 			"message": "m",
 			"author": {
@@ -569,6 +739,26 @@ func TestWorkflowRun_Marshal(t *testing.T) {
 			"id": 1,
 			"url": "u",
 			"name": "n"
+		},
+		"actor": {
+			"login": "l",
+			"id": 1,
+			"avatar_url": "a",
+			"gravatar_id": "g",
+			"name": "n",
+			"company": "c",
+			"blog": "b",
+			"location": "l",
+			"email": "e",
+			"hireable": true,
+			"bio": "b",
+			"twitter_username": "t",
+			"public_repos": 1,
+			"followers": 1,
+			"following": 1,
+			"created_at": ` + referenceTimeStr + `,
+			"suspended_at": ` + referenceTimeStr + `,
+			"url": "u"
 		}
 	}`
 
@@ -588,6 +778,7 @@ func TestWorkflowRuns_Marshal(t *testing.T) {
 				HeadBranch: String("hb"),
 				HeadSHA:    String("hs"),
 				RunNumber:  Int(1),
+				RunAttempt: Int(1),
 				Event:      String("e"),
 				Status:     String("s"),
 				Conclusion: String("c"),
@@ -619,14 +810,16 @@ func TestWorkflowRuns_Marshal(t *testing.T) {
 						},
 					},
 				},
-				CreatedAt:     &Timestamp{referenceTime},
-				UpdatedAt:     &Timestamp{referenceTime},
-				JobsURL:       String("j"),
-				LogsURL:       String("l"),
-				CheckSuiteURL: String("c"),
-				ArtifactsURL:  String("a"),
-				CancelURL:     String("c"),
-				RerunURL:      String("r"),
+				CreatedAt:          &Timestamp{referenceTime},
+				UpdatedAt:          &Timestamp{referenceTime},
+				RunStartedAt:       &Timestamp{referenceTime},
+				JobsURL:            String("j"),
+				LogsURL:            String("l"),
+				CheckSuiteURL:      String("c"),
+				ArtifactsURL:       String("a"),
+				CancelURL:          String("c"),
+				RerunURL:           String("r"),
+				PreviousAttemptURL: String("p"),
 				HeadCommit: &HeadCommit{
 					Message: String("m"),
 					Author: &CommitAuthor{
@@ -657,6 +850,26 @@ func TestWorkflowRuns_Marshal(t *testing.T) {
 					URL:  String("u"),
 					Name: String("n"),
 				},
+				Actor: &User{
+					Login:           String("l"),
+					ID:              Int64(1),
+					AvatarURL:       String("a"),
+					GravatarID:      String("g"),
+					Name:            String("n"),
+					Company:         String("c"),
+					Blog:            String("b"),
+					Location:        String("l"),
+					Email:           String("e"),
+					Hireable:        Bool(true),
+					Bio:             String("b"),
+					TwitterUsername: String("t"),
+					PublicRepos:     Int(1),
+					Followers:       Int(1),
+					Following:       Int(1),
+					CreatedAt:       &Timestamp{referenceTime},
+					SuspendedAt:     &Timestamp{referenceTime},
+					URL:             String("u"),
+				},
 			},
 		},
 	}
@@ -671,6 +884,7 @@ func TestWorkflowRuns_Marshal(t *testing.T) {
 				"head_branch": "hb",
 				"head_sha": "hs",
 				"run_number": 1,
+				"run_attempt": 1,
 				"event": "e",
 				"status": "s",
 				"conclusion": "c",
@@ -704,12 +918,14 @@ func TestWorkflowRuns_Marshal(t *testing.T) {
 				],
 				"created_at": ` + referenceTimeStr + `,
 				"updated_at": ` + referenceTimeStr + `,
+				"run_started_at": ` + referenceTimeStr + `,
 				"jobs_url": "j",
 				"logs_url": "l",
 				"check_suite_url": "c",
 				"artifacts_url": "a",
 				"cancel_url": "c",
 				"rerun_url": "r",
+				"previous_attempt_url": "p",
 				"head_commit": {
 					"message": "m",
 					"author": {
@@ -739,7 +955,27 @@ func TestWorkflowRuns_Marshal(t *testing.T) {
 					"id": 1,
 					"url": "u",
 					"name": "n"
-				}
+				},
+				"actor": {
+					"login": "l",
+					"id": 1,
+					"avatar_url": "a",
+					"gravatar_id": "g",
+					"name": "n",
+					"company": "c",
+					"blog": "b",
+					"location": "l",
+					"email": "e",
+					"hireable": true,
+					"bio": "b",
+					"twitter_username": "t",
+					"public_repos": 1,
+					"followers": 1,
+					"following": 1,
+					"created_at": ` + referenceTimeStr + `,
+					"suspended_at": ` + referenceTimeStr + `,
+					"url": "u"
+				}		
 			}
 		]
 	}`
@@ -840,3 +1076,47 @@ func TestWorkflowRunUsage_Marshal(t *testing.T) {
 
 	testJSONMarshal(t, u, want)
 }
+
+func TestActionService_PendingDeployments(t *testing.T) {
+	client, mux, _, teardown := setup()
+	defer teardown()
+
+	input := &PendingDeploymentsRequest{EnvironmentIDs: []int64{3, 4}, State: "approved", Comment: ""}
+
+	mux.HandleFunc("/repos/o/r/actions/runs/399444496/pending_deployments", func(w http.ResponseWriter, r *http.Request) {
+		v := new(PendingDeploymentsRequest)
+		json.NewDecoder(r.Body).Decode(v)
+
+		testMethod(t, r, "POST")
+		if !cmp.Equal(v, input) {
+			t.Errorf("Request body = %+v, want %+v", v, input)
+		}
+
+		fmt.Fprint(w, `[{"id":1}, {"id":2}]`)
+	})
+
+	ctx := context.Background()
+	deployments, _, err := client.Actions.PendingDeployments(ctx, "o", "r", 399444496, input)
+	if err != nil {
+		t.Errorf("Actions.PendingDeployments returned error: %v", err)
+	}
+
+	want := []*Deployment{{ID: Int64(1)}, {ID: Int64(2)}}
+	if !cmp.Equal(deployments, want) {
+		t.Errorf("Actions.PendingDeployments returned %+v, want %+v", deployments, want)
+	}
+
+	const methodName = "PendingDeployments"
+	testBadOptions(t, methodName, func() (err error) {
+		_, _, err = client.Actions.PendingDeployments(ctx, "\n", "\n", 399444496, input)
+		return err
+	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		got, resp, err := client.Actions.PendingDeployments(ctx, "o", "r", 399444496, input)
+		if got != nil {
+			t.Errorf("testNewRequestAndDoFailure %v = %#v, want nil", methodName, got)
+		}
+		return resp, err
+	})
+}
