@@ -132,6 +132,14 @@ func TestOrganizationsService_GetHook(t *testing.T) {
 		_, _, err = client.Organizations.GetHook(ctx, "\n", -1)
 		return err
 	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		got, resp, err := client.Organizations.GetHook(ctx, "o", 1)
+		if got != nil {
+			t.Errorf("testNewRequestAndDoFailure %v = %#v, want nil", methodName, got)
+		}
+		return resp, err
+	})
 }
 
 func TestOrganizationsService_GetHook_invalidOrg(t *testing.T) {
@@ -177,6 +185,14 @@ func TestOrganizationsService_EditHook(t *testing.T) {
 		_, _, err = client.Organizations.EditHook(ctx, "\n", -1, input)
 		return err
 	})
+
+	testNewRequestAndDoFailure(t, methodName, client, func() (*Response, error) {
+		got, resp, err := client.Organizations.EditHook(ctx, "o", 1, input)
+		if got != nil {
+			t.Errorf("testNewRequestAndDoFailure %v = %#v, want nil", methodName, got)
+		}
+		return resp, err
+	})
 }
 
 func TestOrganizationsService_EditHook_invalidOrg(t *testing.T) {
