@@ -9,7 +9,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"net/url"
 	"testing"
@@ -146,7 +146,7 @@ func TestRepositoriesService_DownloadContents_Success(t *testing.T) {
 		t.Errorf("Repositories.DownloadContents returned status code %v, want %v", got, want)
 	}
 
-	bytes, err := ioutil.ReadAll(r)
+	bytes, err := io.ReadAll(r)
 	if err != nil {
 		t.Errorf("Error reading response body: %v", err)
 	}
@@ -198,7 +198,7 @@ func TestRepositoriesService_DownloadContents_FailedResponse(t *testing.T) {
 		t.Errorf("Repositories.DownloadContents returned status code %v, want %v", got, want)
 	}
 
-	bytes, err := ioutil.ReadAll(r)
+	bytes, err := io.ReadAll(r)
 	if err != nil {
 		t.Errorf("Error reading response body: %v", err)
 	}
@@ -276,7 +276,7 @@ func TestRepositoriesService_DownloadContentsWithMeta_Success(t *testing.T) {
 		t.Errorf("Repositories.DownloadContentsWithMeta returned status code %v, want %v", got, want)
 	}
 
-	bytes, err := ioutil.ReadAll(r)
+	bytes, err := io.ReadAll(r)
 	if err != nil {
 		t.Errorf("Error reading response body: %v", err)
 	}
@@ -339,7 +339,7 @@ func TestRepositoriesService_DownloadContentsWithMeta_FailedResponse(t *testing.
 		t.Errorf("Repositories.DownloadContentsWithMeta returned status code %v, want %v", got, want)
 	}
 
-	bytes, err := ioutil.ReadAll(r)
+	bytes, err := io.ReadAll(r)
 	if err != nil {
 		t.Errorf("Error reading response body: %v", err)
 	}
@@ -670,12 +670,12 @@ func TestRepositoriesService_DeleteFile(t *testing.T) {
 func TestRepositoriesService_GetArchiveLink(t *testing.T) {
 	client, mux, _, teardown := setup()
 	defer teardown()
-	mux.HandleFunc("/repos/o/r/tarball", func(w http.ResponseWriter, r *http.Request) {
+	mux.HandleFunc("/repos/o/r/tarball/yo", func(w http.ResponseWriter, r *http.Request) {
 		testMethod(t, r, "GET")
 		http.Redirect(w, r, "http://github.com/a", http.StatusFound)
 	})
 	ctx := context.Background()
-	url, resp, err := client.Repositories.GetArchiveLink(ctx, "o", "r", Tarball, &RepositoryContentGetOptions{}, true)
+	url, resp, err := client.Repositories.GetArchiveLink(ctx, "o", "r", Tarball, &RepositoryContentGetOptions{Ref: "yo"}, true)
 	if err != nil {
 		t.Errorf("Repositories.GetArchiveLink returned error: %v", err)
 	}
@@ -744,6 +744,23 @@ func TestRepositoriesService_GetArchiveLink_StatusMovedPermanently_followRedirec
 	}
 }
 
+func TestRepositoriesService_GetArchiveLink_invalidLocationHeader(t *testing.T) {
+	client, mux, _, teardown := setup()
+	defer teardown()
+
+	mux.HandleFunc("/repos/o/r/tarball", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, "GET")
+		ctlChar := 0x7f
+		badURL := "https://google.com" + string(byte(ctlChar))
+		w.Header().Add("Location", badURL)
+		w.WriteHeader(http.StatusFound)
+	})
+
+	ctx := context.Background()
+	_, _, err := client.Repositories.GetArchiveLink(ctx, "o", "r", Tarball, &RepositoryContentGetOptions{}, false)
+	testURLParseError(t, err)
+}
+
 func TestRepositoriesService_GetContents_NoTrailingSlashInDirectoryApiPath(t *testing.T) {
 	client, mux, _, teardown := setup()
 	defer teardown()
