@@ -15,7 +15,7 @@ import (
 	"net/http"
 	"os"
 
-	"github.com/google/go-github/v38/github"
+	"github.com/google/go-github/v48/github"
 	"golang.org/x/oauth2"
 )
 
@@ -39,17 +39,6 @@ func init() {
 		client = github.NewClient(tc)
 		auth = true
 	}
-
-	// Environment variables required for Authorization integration tests
-	vars := []string{envKeyGitHubUsername, envKeyGitHubPassword, envKeyClientID, envKeyClientSecret}
-
-	for _, v := range vars {
-		value := os.Getenv(v)
-		if value == "" {
-			print("!!! " + fmt.Sprintf(msgEnvMissing, v) + " !!!\n\n")
-		}
-	}
-
 }
 
 func checkAuth(name string) bool {
@@ -60,6 +49,18 @@ func checkAuth(name string) bool {
 }
 
 func createRandomTestRepository(owner string, autoinit bool) (*github.Repository, error) {
+	// determine the owner to use if one wasn't specified
+	if owner == "" {
+		owner = os.Getenv("GITHUB_OWNER")
+		if owner == "" {
+			me, _, err := client.Users.Get(context.Background(), "")
+			if err != nil {
+				return nil, err
+			}
+			owner = *me.Login
+		}
+	}
+
 	// create random repo name that does not currently exist
 	var repoName string
 	for {
@@ -76,7 +77,14 @@ func createRandomTestRepository(owner string, autoinit bool) (*github.Repository
 	}
 
 	// create the repository
-	repo, _, err := client.Repositories.Create(context.Background(), "", &github.Repository{Name: github.String(repoName), AutoInit: github.Bool(autoinit)})
+	repo, _, err := client.Repositories.Create(
+		context.Background(),
+		owner,
+		&github.Repository{
+			Name:     github.String(repoName),
+			AutoInit: github.Bool(autoinit),
+		},
+	)
 	if err != nil {
 		return nil, err
 	}
