@@ -233,7 +233,7 @@ type fakeDocCache struct {
 	endpoints endpointsByFragmentID
 }
 
-func (f *fakeDocCache) UrlByMethodAndPath(methodAndPath string) (string, bool) {
+func (f *fakeDocCache) URLByMethodAndPath(methodAndPath string) (string, bool) {
 	f.t.Helper()
 	for fragmentID, endpoints := range f.endpoints {
 		for _, endpoint := range endpoints {
@@ -241,17 +241,18 @@ func (f *fakeDocCache) UrlByMethodAndPath(methodAndPath string) (string, bool) {
 				key := fmt.Sprintf("%v %v", endpoint.httpMethod, urlFormat)
 				if key == methodAndPath {
 					url := fmt.Sprintf("%v#%v", f.baseURL, fragmentID)
-					// log.Printf("UrlByMethodAndPath(%q) = (%q, true)", methodAndPath, url)
+					// log.Printf("URLByMethodAndPath(%q) = (%q, true)", methodAndPath, url)
 					return url, true
 				}
 			}
 		}
 	}
-	f.t.Fatalf("fakeDocCache.UrlByMethodAndPath: unable to find method %v", methodAndPath)
+	f.t.Fatalf("fakeDocCache.URLByMethodAndPath: unable to find method %v", methodAndPath)
 	return "", false
 }
 
-func (f *fakeDocCache) CacheDocFromInternet(url, filename string) {} // no-op
+func (f *fakeDocCache) CacheDocFromInternet(url, filename string, pos token.Position) {}                          // no-op
+func (f *fakeDocCache) Position(pos token.Pos) token.Position                         { return token.Position{} } // no-op
 
 // fakeFileRewriter implements FileRewriter.
 type fakeFileRewriter struct {
@@ -433,6 +434,10 @@ func TestSortAndMergeFileEdits(t *testing.T) {
 		},
 	}
 
+	fileEditEqual := cmp.Comparer(func(a, b *FileEdit) bool {
+		return a.fromText == b.fromText && a.pos == b.pos && a.toText == b.toText
+	})
+
 	for i, tt := range tests {
 		t.Run(fmt.Sprintf("test #%v: %v", i, tt.name), func(t *testing.T) {
 			got := sortAndMergeFileEdits(tt.fileEdits)
@@ -445,7 +450,7 @@ func TestSortAndMergeFileEdits(t *testing.T) {
 				if i < len(tt.want) {
 					wantFileEdit = tt.want[i]
 				}
-				if !cmp.Equal(got[i], wantFileEdit) {
+				if !cmp.Equal(got[i], wantFileEdit, fileEditEqual) {
 					t.Errorf("got[%v] =\n%#v\nwant[%v]:\n%#v", i, got[i], i, wantFileEdit)
 				}
 			}
@@ -505,39 +510,50 @@ func TestPerformBufferEdits(t *testing.T) {
 
 func TestGitURL(t *testing.T) {
 	tests := []struct {
-		name string
-		s    string
-		want string
+		name     string
+		s        string
+		wantBase string
+		wantFull string
 	}{
 		{name: "empty string"},
 		{name: "non-http", s: "howdy"},
 		{
-			name: "normal URL, no slash",
-			s:    "https://docs.github.com/en/free-pro-team@latest/rest/reference/activity/events",
-			want: "https://docs.github.com/en/free-pro-team@latest/rest/reference/activity/events/",
+			name:     "normal URL, no slash",
+			s:        "https://docs.github.com/en/rest/activity/events",
+			wantBase: "https://docs.github.com/en/rest/activity/events/",
+			wantFull: "https://docs.github.com/en/rest/activity/events/",
 		},
 		{
-			name: "normal URL, with slash",
-			s:    "https://docs.github.com/en/free-pro-team@latest/rest/reference/activity/events/",
-			want: "https://docs.github.com/en/free-pro-team@latest/rest/reference/activity/events/",
+			name:     "normal URL, with slash",
+			s:        "https://docs.github.com/en/rest/activity/events/",
+			wantBase: "https://docs.github.com/en/rest/activity/events/",
+			wantFull: "https://docs.github.com/en/rest/activity/events/",
 		},
 		{
-			name: "normal URL, with fragment identifier",
-			s:    "https://docs.github.com/en/free-pro-team@latest/rest/reference/activity/events/#list-public-events",
-			want: "https://docs.github.com/en/free-pro-team@latest/rest/reference/activity/events/",
+			name:     "normal URL, with fragment identifier",
+			s:        "https://docs.github.com/en/rest/activity/events/#list-public-events",
+			wantBase: "https://docs.github.com/en/rest/activity/events/",
+			wantFull: "https://docs.github.com/en/rest/activity/events/#list-public-events",
 		},
 	}
 
 	for i, tt := range tests {
 		t.Run(fmt.Sprintf("test #%v: %v", i, tt.name), func(t *testing.T) {
-			got := getURL(tt.s)
-			if got != tt.want {
-				t.Errorf("getURL = %v ; want %v", got, tt.want)
+			gotBase, gotFull := getURL(tt.s)
+
+			if gotBase != tt.wantBase {
+				t.Errorf("getURL base = %v ; want %v", gotBase, tt.wantBase)
+			}
+
+			if gotFull != tt.wantFull {
+				t.Errorf("getURL full = %v ; want %v", gotFull, tt.wantFull)
 			}
 		})
 	}
 }
 
+var endpointsEqualCmp = cmp.Comparer(endpointsEqual)
+
 func testWebPageHelper(t *testing.T, got, want map[string][]*Endpoint) {
 	t.Helper()
 
@@ -546,16 +562,19 @@ func testWebPageHelper(t *testing.T, got, want map[string][]*Endpoint) {
 		if len(got[k]) != len(w) {
 			t.Errorf("len(got[%q]) = %v, len(want[%q]) = %v", k, len(got[k]), k, len(w))
 		}
+
 		for i := 0; i < len(got[k]); i++ {
 			var wantEndpoint *Endpoint
 			if ok && i < len(w) {
 				wantEndpoint = w[i]
 			}
-			if !cmp.Equal(got[k][i], wantEndpoint) {
-				t.Errorf("got[%q][%v] =\n%#v\nwant[%q][%v]:\n%#v", k, i, got[k][i], k, i, wantEndpoint)
+
+			if diff := cmp.Diff(wantEndpoint, got[k][i], endpointsEqualCmp); diff != "" {
+				t.Errorf("endpoint %q i=%v mismatch (-want +got):\n%v", k, i, diff)
 			}
 		}
 	}
+
 	for k := range want {
 		if _, ok := got[k]; !ok {
 			t.Errorf("got[%q] = nil\nwant[%q]:\n%#v", k, k, want[k])
@@ -590,7 +609,7 @@ func TestParseEndpoint(t *testing.T) {
 		t.Run(fmt.Sprintf("test #%v: %v", i, tt.name), func(t *testing.T) {
 			got := parseEndpoint(tt.s, tt.method)
 
-			if !cmp.Equal(got, tt.want) {
+			if !cmp.Equal(got, tt.want, endpointsEqualCmp) {
 				t.Errorf("parseEndpoint = %#v, want %#v", got, tt.want)
 			}
 		})
